#include <iostream>
#include <algorithm>
#include <vector>
#include <fstream>

using namespace std;

struct coord
{
	int row;
	int col;
	coord()
	{
		row = -1;
		col = -1;
	}
};

struct animal
{
	int position;
	int** shape;
	int rows, cols;
	int force;
	int minSafety;
	void init(int r,int c,int k, int t, int p)
	{
		shape = new int*[r];
		for(int i = 0; i < r; i++)
		{
			shape[i] = new int[c];
		}
		rows = r;
		cols = c;
		force = k;
		minSafety = t;
		position = p;
	}
	
	~animal()
	{
		for(int i = 0; i < rows; i++)
		{
			delete[] shape[i];
		}
		delete[] shape;
	}
};

void input(int& N, int& M, int& P, int** field, animal* animals)
{
    for(int i = 0; i < N; i++)
    {
        for(int k = 0; k < M; k++)
        {
            cin>>field[i][k];
        }
    }

	char* line = new char[50];
    for(int i = 0; i < P; i++)
    {
		int r, c, k, t;
		
        cin>>r>>c>>k>>t;
		animals[i].init(r, c, k, t, i+1);
		for(int j = 0; j < r; j++)
		{
			cin >> line;
			for(int z = 0; z < c; z++)
			{
				animals[i].shape[j][z] = (line[z] == '1') ? 1 : 0;
			}
		}
    }
	delete [] line;
}

bool sortByForce(animal& animal1, animal& animal2)
{
	return animal1.force !=  animal2.force ?
		animal1.force < animal2.force : animal1.rows * animal1.cols > animal2.rows * animal2.cols;
}

coord checkCoords(animal& curAnimal, int** field, int N, int M)
{
	int sum = 0, maxSum = 0;
	coord topLeft = coord();
	for(int i = 0; i <= N - curAnimal.rows; i++)
	{
		for(int k = 0; k <= M - curAnimal.cols; k++)
		{
			int j;
			for(j = 0; j < curAnimal.rows; j++)
			{
				int l;
				for(l = 0; l < curAnimal.cols; l++)
				{
					if(curAnimal.shape[j][l] == 1 && field[i + j][k + l] < curAnimal.minSafety)
					{
						break;
					}
					if(curAnimal.shape[j][l] == 1)
					{
						sum += field[i + j][k + l];
					}
				}
				if(l != curAnimal.cols)
				{
					break;
				}
			}
			if(sum >= maxSum)
			{
				maxSum = sum;
				topLeft.row = i;
				topLeft.col = k;
			}
			sum = 0;
		}
	}
	return topLeft;
}

void placeAnimal(coord coords, animal curAnimal, int** field)
{
	for(int i = 0; i < curAnimal.rows; i++)
	{
		for(int k = 0; k < curAnimal.rows; k++)
		{
			if(curAnimal.shape[i][k] == 1)
			{
				field[i + coords.row][k + coords.col] /= curAnimal.force;
			}
		}
	}
}

int main()
{
	int M, N, P, animalsJumped = 0;
	vector<coord> jumperCoords;
	vector<int> jumperPosition;
	int** field;
    cin>>N>>M>>P;
	animal* animals = new animal[P];
    field = new int*[M];
    for(int i = 0; i < N;i++)
    {
        field[i] = new int[N];
    }
    input(N, M, P, field, animals);

	//sort(animals, animals+P, &sortByForce);

	for(int i = 0; i < P; i++)
	{
		coord bestCoords = checkCoords(animals[i], field, N, M);
		if(bestCoords.row == -1 || bestCoords.col == -1)
		{
			continue;
		}
		placeAnimal(bestCoords, animals[i], field);
		jumperCoords.push_back(bestCoords);
		jumperPosition.push_back(animals[i].position);
		animalsJumped++;
	}

	ofstream parashOut;
	parashOut.open("parash.out");
	
	parashOut<<animalsJumped<<endl;
	for(int i = 0; i < animalsJumped; i++)
	{
		parashOut<<jumperPosition[i]<<' '<<jumperCoords[i].row+1<<' '<<jumperCoords[i].col+1<<endl;
	}
	parashOut.close();

    return 0;
}